@@ -26,8 +26,14 @@ import asyncio
 import time
 from starlette.concurrency import run_in_threadpool
 from typing import List
-
+import brave.api.service.container_service as container_service
 from brave.app_container import AppContainer
+import brave.api.service.notebook as notebook_service
+import shutil
+from fastapi import  File, UploadFile
+import shutil
+import time
+
 
 pipeline = APIRouter()
 
@@ -197,6 +203,7 @@ async def get_component_parent(component_id,component_type):
         t_pipeline_components.c.content,
         t_pipeline_components.c.namespace,
         t_pipeline_components.c.component_name,
+        t_pipeline_components.c.tags,
         t_namespace.c.name.label("namespace_name"),
         cast(null(), String(255)).label("relation_type"),
         cast(null(), String(255)).label("parent_component_id"),
@@ -222,6 +229,7 @@ async def get_component_parent(component_id,component_type):
         tp1.c.content,
         tp1.c.namespace,
         tp1.c.component_name,
+        tp1.c.tags,
         tn1.c.name.label("namespace_name"),
         rel.c.relation_type,
         rel.c.parent_component_id,
@@ -250,10 +258,10 @@ async def get_component_parent(component_id,component_type):
         del child_item["content"]
 
     parent_item_list = [dict(item) for item in data if item['component_type'] != component_type]
-    resul_dict= {}
-    resul_dict['script'] = dict(child_item)
-    resul_dict['parent'] = parent_item_list
-    return resul_dict
+    # resul_dict= {}
+    # resul_dict['script'] = dict(child_item)
+    child_item['parent'] = parent_item_list
+    return child_item
 
 
 @pipeline.get("/get-pipeline-dag/{pipeline_id}",tags=['pipeline'])
@@ -324,7 +332,7 @@ async def get_module_content(component_id,script_name:ScriptName):
         find_component = pipeline_service.find_pipeline_by_id(conn,component_id)
         find_component = {
             **{k:v for k,v in find_component.items() if k != "content"},
-            **json.loads(find_component.content)
+            **json.loads(find_component["content"])
         }
         if not find_component:
             raise HTTPException(status_code=500, detail=f"根据{component_id}不能找到记录!")
@@ -577,8 +585,8 @@ async def save_pipeline_relation(conn,savePipelineRelation):
     if savePipelineRelation.parent_component_id:
         parent_component = pipeline_service.find_pipeline_by_id(conn,savePipelineRelation.parent_component_id)
         if parent_component:
-            save_pipeline_relation_dict['namespace'] = parent_component.namespace
-            namespace = parent_component.namespace
+            save_pipeline_relation_dict['namespace'] = parent_component["namespace"]
+            namespace = parent_component["namespace"]
     if savePipelineRelation.relation_id:
         stmt = t_pipeline_components_relation.update().values(save_pipeline_relation_dict).where(t_pipeline_components_relation.c.relation_id==savePipelineRelation.relation_id)
     else:
@@ -698,10 +706,15 @@ async def delete_component(component_id: str):
         if  parent_find_pipeine or child_find_pipeine:
             raise HTTPException(status_code=500, detail=f"不能删除存在关联!") 
         else:
+            find_component = pipeline_service.find_component_by_id(conn,component_id)
             stmt = t_pipeline_components.delete().where(t_pipeline_components.c.component_id == component_id)
             conn.execute(stmt)
             pipeline_service.delete_wrap_pipeline_dir(component_id)
-            return {"message":"success"}
+
+    with get_engine().begin() as conn:   
+        pipeline_service.write_all_component(conn,find_component["namespace"])
+        pipeline_service.write_all_component_relation(conn,find_component["namespace"])
+    return {"message":"success"}
 
 @pipeline.get("/find-by-component-id/{component_id}",tags=['pipeline'])
 async def find_by_components_id(component_id):
@@ -717,6 +730,7 @@ async def import_namespace_component(namespace:str,force:bool=False):
         pipeline_service.import_component(conn,namespace,force)
         pipeline_service.import_component_relation(conn,namespace,force)
         namespace_service.import_namespace(conn,namespace,force)
+        container_service.import_container(conn,namespace,force)
     return {"message":"success"}
 
 def get_namespace_by_file(file):
@@ -743,7 +757,7 @@ async def get_depend_component(component_id):
         find_component = pipeline_service.find_pipeline_by_id(conn, component_id)
         if not find_component:
             raise HTTPException(status_code=404, detail=f"Component {component_id} not found")
-        namespace = find_component.namespace
+        namespace = find_component['namespace']
         child_depend_component = pipeline_service.get_child_depend_component(conn, namespace, component_id)
         parent_depend_component = pipeline_service.get_parent_depend_component(conn, namespace, component_id)
         return list(child_depend_component) + list(parent_depend_component)
@@ -756,5 +770,80 @@ async def save_component_relation_order(saveOrder:list[SaveOrder]):
     return {"message":"success"}
 
 
+@pipeline.post("/update-component-description/{component_id}",tags=['pipeline'])
+async def update_component_description(component_id,description ):
+    with get_engine().begin() as conn:
+        pipeline_service.update_component_description(conn,component_id,description)
+    return {"message":"success"}
 
 
+
+
+
+@pipeline.post("/component/convert-ipynb/{component_id}",tags=['pipeline'])
+async def convert_ipynb(component_id):
+    # module_dir = queryModule.component_id
+    with get_engine().begin() as conn:
+        find_component = pipeline_service.find_pipeline_by_id(conn,component_id)
+        find_component = {
+            **{k:v for k,v in find_component.items() if k != "content"},
+            **json.loads(find_component["content"])
+        }
+        if not find_component:
+            raise HTTPException(status_code=500, detail=f"根据{component_id}不能找到记录!")
+    module_info:dict = pipeline_service.find_component_module(find_component, ScriptName.main)
+    script_path  = module_info['path']
+    ipynb_path = os.path.dirname(script_path)
+    ipynb_path = f"{ipynb_path}/main.ipynb"
+    if os.path.exists(script_path) and os.path.exists(ipynb_path):
+        shutil.copy(script_path, f"{script_path}.tmp")
+        notebook_service.convert_notebook(ipynb_path, script_path)
+        return "success"
+    raise HTTPException(status_code=404, detail=f"{script_path}或{ipynb_path}不存在!")
+
+
+# @analysis_api.post("/analysis/convert-ipynb/{analysis_id}")
+# async def convert_ipynb(analysis_id):
+#     with get_engine().begin() as conn:
+#         find_analysis = analysis_service.find_analysis_by_id(conn,analysis_id)
+#     script_path  = find_analysis["pipeline_script"]
+#     ipynb_path = os.path.dirname(script_path)
+#     ipynb_path = f"{ipynb_path}/main.ipynb"
+#     if os.path.exists(script_path) and os.path.exists(ipynb_path):
+#         shutil.copy(script_path, f"{script_path}.tmp")
+#         notebook_service.convert_notebook(ipynb_path, script_path)
+#         return "success"
+#     raise HTTPException(status_code=404, detail=f"{script_path}或{ipynb_path}不存在!")
+
+
+@pipeline.post("/component/upload/{component_id}")
+async def upload_image(component_id,file: UploadFile = File(...)):
+    # 限制只能上传图片类型
+    if not file.content_type.startswith("image/"):
+        raise HTTPException(status_code=404, detail=f"只允许上传图片文件!")
+    with get_engine().begin() as conn:
+        find_component = pipeline_service.find_pipeline_by_id(conn,component_id)
+        find_component = {
+            **{k:v for k,v in find_component.items() if k != "content"},
+            **json.loads(find_component["content"])
+        }
+        if not find_component:
+            raise HTTPException(status_code=500, detail=f"根据{component_id}不能找到记录!")
+    module_info:dict = pipeline_service.find_component_module(find_component, ScriptName.main)
+    script_path  = module_info['path']
+    file_path = os.path.dirname(script_path)
+    name, ext = os.path.splitext(file.filename)
+
+    # file_path = os.path.join(file_path, file.filename)
+    filename = f"main{ext}"
+    file_path = f"{file_path}/{filename}"
+
+    # 保存文件到本地
+    with open(file_path, "wb") as buffer:
+        shutil.copyfileobj(file.file, buffer)
+    settings = get_settings()
+    url_suffix = file_path.replace(str(settings.PIPELINE_DIR),"")
+    ts_str = str(int(time.time()))
+
+    url = f"/brave-api/pipeline-dir{url_suffix}?v={ts_str}"
+    return {"filename": filename, "url":url}
