@@ -13,12 +13,12 @@ from brave.api.schemas.pipeline import PagePipelineQuery, SavePipeline,Pipeline,
 from brave.api.config.db import get_engine
 from brave.api.models.core import t_namespace, t_pipeline_components, t_pipeline_components_edges, t_pipeline_components_relation
 import importlib.resources as resources
-from sqlalchemy import delete, select, and_, join, func,insert,update
+from sqlalchemy import delete, select, and_, join, func,insert,update,or_
 from datetime import datetime
 from brave.api.enum.component_script import ScriptName
 from sqlalchemy import  Column, Integer, String, Text, select, cast, null,text,case
-
-
+from .notebook import generate_notebook
+import brave.api.service.container_service as container_service
 
 def get_pipeline_dir():
     settings = get_settings()
@@ -49,7 +49,9 @@ def find_component_module(component,script_name:ScriptName) -> dict:
         elif script_type == "shell":
             file_type = "sh"
         elif script_type == "r":
-            file_type = "r"
+            file_type = "R"
+        elif script_type == "jupyter":
+            file_type = "ipynb"
         else:
             raise HTTPException(status_code=500, detail=f"script_type {script_type} not found!")
     else:
@@ -177,79 +179,112 @@ def create_file(namespace,component_id,component_type,file_type):
 
     content_text = ""
     if file_type == "py":
-        with resources.files("brave.templete").joinpath(f"{file_type}.py").open("r") as f:
+        with resources.files("brave.templete").joinpath(f"py_plot.py").open("r") as f:
             content_text = f.read()
     elif file_type == "nf":
         with resources.files("brave.templete").joinpath("nextflow.nf").open("r") as f:
             content_text = f.read()
+    elif file_type == "R":
+        with resources.files("brave.templete").joinpath("py_plot.R").open("r") as f:
+            content_text = f.read()
+    # if component_type == "pipeline":
+    analysis_file = f"{pipeline_dir}/{component_type}/{component_id}/main.{file_type}"
+    if not os.path.exists(analysis_file):
+        dir_ = os.path.dirname(analysis_file)
+        if not os.path.exists(dir_):
+            os.makedirs(dir_) 
+        if file_type=="ipynb":
+            generate_notebook(analysis_file)
+        else:
+            with open(analysis_file,"w") as f:
+                f.write(content_text)
+            ipynb_path = os.path.dirname(analysis_file)
+            generate_notebook(f"{ipynb_path}/main.ipynb")
     
-    if component_type == "pipeline":
-        analysisPipline = f"{pipeline_dir}/pipeline/{component_id}/main.{file_type}"
-        # pipelinieJson = f"{pipeline_dir}/main.json"
-        if not os.path.exists(analysisPipline):
-            dir_ = os.path.dirname(analysisPipline)
-            if not os.path.exists(dir_):
-                os.makedirs(dir_) 
-            with open(analysisPipline,"w") as f:
-                f.write("")
-        return analysisPipline
+    return analysis_file
         # if not os.path.exists(parseAnalysisModule):
         #     with open(parseAnalysisModule,"w") as f:
         #         f.write("")
 
-    if component_type == "software":
-        # parseAnalysisModule = f"{pipeline_dir}/software/{component_id}/main.py"
-        analysisPipline = f"{pipeline_dir}/software/{component_id}/main.{file_type}"
-        # dir_list = [parseAnalysisModule,analysisPipline]
-        # for item in dir_list:
-        dir_ = os.path.dirname(analysisPipline)
-        if not os.path.exists(dir_):
-            os.makedirs(dir_) 
+    # if component_type == "software":
+    #     # parseAnalysisModule = f"{pipeline_dir}/software/{component_id}/main.py"
+    #     analysisPipline = f"{pipeline_dir}/software/{component_id}/main.{file_type}"
+    #     # dir_list = [parseAnalysisModule,analysisPipline]
+    #     # for item in dir_list:
+    #     dir_ = os.path.dirname(analysisPipline)
+    #     if not os.path.exists(dir_):
+    #         os.makedirs(dir_) 
         
-        if not os.path.exists(analysisPipline):
+    #     if not os.path.exists(analysisPipline):
   
-            with open(analysisPipline,"w") as f:
-                f.write(content_text)
-        return analysisPipline
-        # if not os.path.exists(parseAnalysisModule):
-        #     with resources.files("brave.templete").joinpath("py_parse_analysis.py").open("r") as f:
-        #         content_text = f.read()
-        #     with open(parseAnalysisModule,"w") as f:
-        #         f.write(content_text)
+    #         with open(analysisPipline,"w") as f:
+    #             f.write(content_text)
+    #     return analysisPipline
+    #     # if not os.path.exists(parseAnalysisModule):
+    #     #     with resources.files("brave.templete").joinpath("py_parse_analysis.py").open("r") as f:
+    #     #         content_text = f.read()
+    #     #     with open(parseAnalysisModule,"w") as f:
+    #     #         f.write(content_text)
         
-        # if "parseAnalysisResultModule" in  content:
-        #     parseAnalysisResultModule = content['parseAnalysisResultModule']
-        #     for item in parseAnalysisResultModule:
-        #         item_file = f"{pipeline_dir}/software/{component_id}/{item['module']}.py"
-        #         dir_ = os.path.dirname(item_file)
-        #         if not os.path.exists(dir_):
-        #             os.makedirs(dir_) 
-        #         if not os.path.exists(item_file):
-        #             with resources.files("brave.templete").joinpath("py_parse_analysis_result.py").open("r") as f:
-        #                 content_text = f.read()
-        #             with open(item_file,"w") as f:
-        #                 f.write(content_text)
+    #     # if "parseAnalysisResultModule" in  content:
+    #     #     parseAnalysisResultModule = content['parseAnalysisResultModule']
+    #     #     for item in parseAnalysisResultModule:
+    #     #         item_file = f"{pipeline_dir}/software/{component_id}/{item['module']}.py"
+    #     #         dir_ = os.path.dirname(item_file)
+    #     #         if not os.path.exists(dir_):
+    #     #             os.makedirs(dir_) 
+    #     #         if not os.path.exists(item_file):
+    #     #             with resources.files("brave.templete").joinpath("py_parse_analysis_result.py").open("r") as f:
+    #     #                 content_text = f.read()
+    #     #             with open(item_file,"w") as f:
+    #     #                 f.write(content_text)
 
-    if component_type == "script":
+    # if component_type == "script":
+   
 
-        py_plot = f"{pipeline_dir}/script/{component_id}/main.{file_type}"
-        py_plot_dir = os.path.dirname(py_plot)
-        if not os.path.exists(py_plot):
-            if not os.path.exists(py_plot_dir):
-                os.makedirs(py_plot_dir)
+    #     py_plot = f"{pipeline_dir}/script/{component_id}/main.{file_type}"
+    #     py_plot_dir = os.path.dirname(py_plot)
+    #     if not os.path.exists(py_plot):
+    #         if not os.path.exists(py_plot_dir):
+    #             os.makedirs(py_plot_dir)
   
-            with open(py_plot,"w") as f:
-                f.write(content_text)
-        return py_plot
-    raise HTTPException(status_code=500, detail=f"component_type {component_type} not create file!")
+    #         with open(py_plot,"w") as f:
+    #             f.write(content_text)
+
+    #     nb_file = f"{pipeline_dir}/script/{component_id}/main.ipynb"
+    #     if not os.path.exists(nb_file):
+    #         generate_notebook(nb_file)
+
+    #     return py_plot
+    # raise HTTPException(status_code=500, detail=f"component_type {component_type} not create file!")
 
 
-
-def find_pipeline_by_id(conn,component_id):
+def find_component_by_id(conn,component_id):
     stmt = t_pipeline_components.select().where(t_pipeline_components.c.component_id ==component_id)
     find_pipeine = conn.execute(stmt).mappings().first()
     return find_pipeine
 
+def find_pipeline_by_id(conn,component_id):
+    stmt = t_pipeline_components.select().where(t_pipeline_components.c.component_id ==component_id)
+    find_component = conn.execute(stmt).mappings().first()
+    
+    # component_type = find_component["component_type"]
+    if find_component:
+        find_component = dict(find_component)
+        if find_component["container_id"]:
+            find_container = container_service.find_container_by_id(conn,find_component["container_id"])
+            find_component['container'] = find_container
+        if find_component["sub_container_id"]:
+            find_container = container_service.find_container_by_id(conn,find_component["sub_container_id"])
+            find_component['sub_container'] = find_container
+
+    # if find_component and find_component["server_container_id"]:
+    #     find_component = dict(find_component)
+    #     find_container = container_service.find_container_by_id(conn,find_component["container_id"])
+    #     find_component['container'] = find_container
+        # find_component['container_name'] = find_container["name"]
+    return find_component
+
 def find_component_by_parent_id(conn,parent_id,relation_type=None):
     stmt = (
         select(
@@ -294,13 +329,17 @@ def list_pipeline(conn,queryPipeline:QueryPipeline):
     return find_pipeine
 
 def format_pipeline_componnet_one(item):
-    content = json.loads(item['content'])
-    item = {**content,**{k:v for k,v in item.items() if k != 'content'}}
-    if 'img' in item:
-        if not item['img']:
-            item['img'] = f"/brave-api/img/pipeline.jpg"
-        else:
-            item['img'] = f"/brave-api/img/{item['namespace']}/{item['component_type']}/{item['component_id']}/{item['img']}"
+    try:
+        content = json.loads(item['content'])
+        item = {**content,**{k:v for k,v in item.items() if k != 'content'}}
+        if 'img' in item:
+            if not item['img']:
+                item['img'] = f"/brave-api/img/pipeline.jpg"
+    except Exception as e:
+        print("component json error",json.dumps(item,indent=4))
+ 
+        # else:
+        #     item['img'] = f"/brave-api/pipeline-dir/{item['namespace']}/{item['component_type']}/{item['component_id']}/{item['img']}"
     return item
 
 def page_pipeline(conn,query:PagePipelineQuery):
@@ -317,7 +356,17 @@ def page_pipeline(conn,query:PagePipelineQuery):
     conditions = []
     if query.component_type is not None:
         conditions.append(t_pipeline_components.c.component_type == query.component_type)
-    
+
+    if query.keywords:
+        keyword_pattern = f"%{query.keywords}%"
+        conditions.append(
+            or_(
+                t_pipeline_components.c.component_name.ilike(keyword_pattern),
+                t_pipeline_components.c.tags.ilike(keyword_pattern),
+                t_pipeline_components.c.description.ilike(keyword_pattern)
+            )
+        )
+
     stmt = stmt.where(and_(*conditions))
     count_stmt = select(func.count()).select_from(t_pipeline_components).where(and_(*conditions))
 
@@ -398,6 +447,9 @@ def find_component_by_namespace(conn,namespace):
     stmt = t_pipeline_components.select().where(t_pipeline_components.c.namespace == namespace)
     return conn.execute(stmt).mappings().all()
 
+def find_component_by_container_id(conn,container_id):
+    stmt = t_pipeline_components.select().where(t_pipeline_components.c.container_id == container_id)
+    return conn.execute(stmt).mappings().all()
 
 def get_child_depend_component(conn,namespace, component_id):
     
@@ -496,6 +548,8 @@ def get_pipeline_v2(conn,name,component_type="pipeline"):
         t_pipeline_components.c.component_name,
         t_pipeline_components.c.position,
         t_pipeline_components.c.edges,
+        t_pipeline_components.c.tags,
+        t_pipeline_components.c.description,
         t_namespace.c.name.label("namespace_name"),
         cast(null(), String(255)).label("relation_type"),
         cast(null(), String(255)).label("parent_component_id"),
@@ -525,9 +579,11 @@ def get_pipeline_v2(conn,name,component_type="pipeline"):
         tp1.c.content,
         tp1.c.namespace,
         tp1.c.component_name,
-        tn1.c.name.label("namespace_name"),
         tp1.c.position,
+        tp1.c.tags,
         tp1.c.edges,
+        tp1.c.description,
+        tn1.c.name.label("namespace_name"),
         rel.c.relation_type,
         rel.c.parent_component_id,
         rel.c.order_index,
@@ -549,6 +605,7 @@ def get_pipeline_v2(conn,name,component_type="pipeline"):
     )
     # 执行查询
     # with get_engine().begin() as conn:
+    # [dict(item) for item in data]
     data = conn.execute(final_query).mappings().all()
     if  len(data) < 0:
         raise HTTPException(status_code=500, detail=f"{name}没有找到!")  
@@ -576,7 +633,14 @@ def get_pipeline_v2(conn,name,component_type="pipeline"):
     return result
 
 def get_one_data(item):
-    content = json.loads(item["content"])
+    try:
+        content = json.loads(item["content"])
+    except Exception as e:
+        print("component json error",json.dumps(dict(item),indent=4))
+        print("component json error",e)
+        content = {}
+    # end try
+    
     item = {k:v for k,v in item.items() if k!="content"}
     return { **content,**item }
 
@@ -654,4 +718,8 @@ def build_pipeline_structure(id_to_node,children_map,root_item):
     return result
 
 
-  
\ No newline at end of file
+  
+def update_component_description(conn, component_id,description):
+    stmt = t_pipeline_components.update().where(t_pipeline_components.c.component_id ==component_id).values(description=description)
+    conn.execute(stmt)
+    # pass
\ No newline at end of file
